@@ -1,12 +1,22 @@
+#ifndef CONFIG_H
+#define CONFIG_H
+
 /* This file contains global options. */
 
 /* global settings defined at compile time */
 #define PACKET_MAXSIZE 0x10000	/* maximum total raw packet size */
 #define HEADER_MAXSIZE 0x2000	/* maximum size for http header (stream cache) */
+#define RESOURCE_MAXSIZE 0x400	/* maximum size for http resource identifier */
 #define CONNECTION_BUCKETS 100	/* number of buckets in connection hash table */
-
-/* memory limits */
 #define MAX_CONNECTIONS 1000	/* maximum number of simultaneous connections */
 
-/* global settings defined at runtime*/
+/* string definitions */
+#define HTTP_INDEXFILE "index.html"
+
+/* global settings defined at runtime */
 int debug;			/* debugging level */
+struct config {
+	int saveStructured;	/* 1 for URL structure based file saving */
+} cfg;
+
+#endif
