@@ -1,5 +1,5 @@
 /*
- * HTTPSniff v0.2 - HTTP Response Sniffer
+ * HTTPSniff v0.3 - HTTP Response Sniffer
  * conn.c
  * (C)2004-2005 Michael Poppitz
  *
@@ -49,13 +49,15 @@ void connTwin(struct connid *twin, struct connid *cid) {
 
 /* add connection */
 void connAdd(struct connid *cid, u_int32_t seq) {
-	struct connection *conn;
+	struct connection *conn, *twinConn;
+	struct connid twinCid;
 	
 	conn = hashFind(cid);
 	if (!conn) {
+		/* create connection */
 		conn = (struct connection *)malloc(sizeof(struct connection));
 		if (!conn) {
-			perror("connAdd(): Cannot add connection");
+			perror("connAdd(): Cannot allocate memory.");
 			return;
 		}
 		memcpy(&conn->cid, cid, sizeof(struct connid));
@@ -64,9 +66,23 @@ void connAdd(struct connid *cid, u_int32_t seq) {
 		conn->time = time(NULL);
 		conn->size = 0;
 		streamOpen(&conn->stream);
-		if(!hashAdd(conn))
-			printf("connAdd(): OOPS: Connection limit exceeded.\n");
+		if(!hashAdd(&conn->cid, conn)) {
+			perror("connAdd(): Connection limit exceeded.");
+			streamClose(&conn->stream);
+			free(conn);
+			return;
+		}
+		
 		if (debug > 4) connDebug(cid, "Added new connection.");
+	
+		/* check if a twin exists and if so, connect the two */
+		connTwin(&twinCid, cid);
+		twinConn = hashFind(&twinCid);
+		if (twinConn) {
+			twinConn->stream.twin = &conn->stream;
+			conn->stream.twin = &twinConn->stream;
+			if (debug > 4) connDebug(cid, "Found and connected twin.");
+		}
 	} else {
 		conn->time = time(NULL);
 		if (debug > 4) connDebug(cid, "Ignored add. Connection already exists.");
@@ -100,7 +116,7 @@ void connDataInternal(struct connid *cid, u_int32_t seq, u_char *packet, u_int32
 				conn->seq += seg->size;
 				conn->size += seg->size;
 				if (debug > 3) printf("   => segment match: %u bytes (%u total) (new seq: %u)\n", len, conn->size, conn->seq);
-				streamWrite(&conn->stream, seg->data, seg->size);
+				streamWrite(&conn->stream, (char *)seg->data, seg->size);
 			}
 			if (seg->flags & SEGMENT_FLAG_CLOSE) {
 				if (debug > 3) {
