@@ -1,5 +1,5 @@
 /*
- * HTTPSniff v0.2 - HTTP Response Sniffer
+ * HTTPSniff v0.3 - HTTP Response Sniffer
  * main.c
  * (C)2004-2005 Michael Poppitz
  *
@@ -13,11 +13,15 @@
 
 #include "config.h"
 #include "sniff.h"
+#include "nids.h"
 
 #define MODE_AUTO 0
 #define MODE_FILE 1
 #define MODE_DEV 2
 
+#define STACK_INTERNAL 0
+#define STACK_NIDS 1
+
 void mainHelp(char *name) {
 	printf(
 		"\n"
@@ -26,11 +30,16 @@ void mainHelp(char *name) {
 		"Where <options> are:\n"
 		"	-i	force <name> to be treated as network interface\n"
 		"	-f	force <name> to be treated as dump file\n"
+		"\n"
+		"	-x	use builtin relaxed tcp/ip stack instead of libnids (unsafe)\n"
+		"	-u	use URL based file/directory structure for saved files (unsafe)\n"
+		"\n"
 		"	-v	verbose - additional v increase output further\n"
 		"	-q	quiet - additional q decrease output further\n"
 		"	-h	this help page\n"
 		"\n"
-		"The optional <filter> can be any pcap filter program. See tcpdump man page for details.\n"
+		"The optional <filter> can be any pcap filter program. See tcpdump man page for\n"
+		"details.\n"
 		"\n"
 		,
 		name
@@ -39,12 +48,16 @@ void mainHelp(char *name) {
 }
 
 int main(int argc, char **argv) {
-	int i, j, mode, filterSize = 1;
+	int i, j, result, mode, stack, filterSize = 1;
 	char *name = NULL, *filter = NULL;
 	
+	/* set defaults */
 	debug = 1;
+	cfg.saveStructured = 0;
 	mode = MODE_AUTO;
+	stack = STACK_NIDS;
 
+	/* parse arguments */
 	if (argc < 2)
 		mainHelp(argv[0]);
 	
@@ -72,6 +85,14 @@ int main(int argc, char **argv) {
 						mode = MODE_DEV;
 						break;
 
+					case 'x':
+						stack = STACK_INTERNAL;
+						break;
+					
+					case 'u':
+						cfg.saveStructured = 1;
+						break;
+
 					case 'q':
 						if (debug > 0) debug--;
 						break;
@@ -102,15 +123,24 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	sniffInit();
-	
+	// if nids is to be used, check if it was compiled in
+	if (stack == STACK_NIDS && !nidsAvailable())
+		stack = STACK_INTERNAL;
+
+	// perform task
 	if (mode == MODE_AUTO || mode == MODE_FILE) {
 		if (debug > 0) printf("Trying to read from file '%s'...\n", name);
-		if (sniffFile(name, filter)) {
+		
+		if (stack == STACK_INTERNAL) {
+			result = sniffFile(name, filter);
+		} else {
+			result = nidsFile(name, filter);
+		}
+		
+		if (result) {
 			if (debug > 0) printf("Failed.\n");
 			if (mode != MODE_AUTO) return (EXIT_FAILURE);
 		} else {
-			sniffKill();
 			if (debug > 0) printf("Finished.\n");
 			return (EXIT_SUCCESS);
 		}
@@ -118,11 +148,17 @@ int main(int argc, char **argv) {
 		
 	if (mode == MODE_AUTO || mode == MODE_DEV) {
 		if (debug > 0) printf("Trying to read from device '%s'...\n", name);
-		if (sniffDevice(name, filter)) {
+		
+		if (stack == STACK_INTERNAL) {
+			result = sniffDevice(name, filter);
+		} else {
+			result = nidsDevice(name, filter);
+		}
+		
+		if (result) {
 			if (debug > 0) printf("Failed.\n");
 			return (EXIT_FAILURE);
 		} else {
-			sniffKill();
 			if (debug > 0) printf("Finished.\n");
 			return (EXIT_SUCCESS);
 		}
