@@ -1,5 +1,5 @@
 /*
- * HTTPSniff v0.2 - HTTP Response Sniffer
+ * HTTPSniff v0.3 - HTTP Response Sniffer
  * sniff.c
  * (C)2004-2005 Michael Poppitz
  *
@@ -218,8 +218,10 @@ int sniffPcap(int type, char *res, char *filter) {
 
 	/* process packets - abort only if end of save file reached */
 	for (status = 0; status != -2; status = pcap_next_ex(pcap, &hdr, (const u_char **)&packet)) {
-		if (debug > 5) printf("\n*** Read new packet of size %i ***\n", hdr->caplen);
-		handler(packet, hdr->caplen);
+		if (status == 1) {
+			if (debug > 5) printf("\n*** Read new packet of size %i ***\n", hdr->caplen);
+			handler(packet, hdr->caplen);
+		}
 	}
 
 	/* clean up */
@@ -241,9 +243,19 @@ void sniffKill() {
 }
 
 int sniffDevice(char *dev, char *filter) {
-	return (sniffPcap(0, dev, filter));
+	int ret;
+	
+	sniffInit();
+	ret = sniffPcap(0, dev, filter);
+	sniffKill();
+	return (ret);
 }
 
 int sniffFile(char *file, char *filter) {
-	return (sniffPcap(1, file, filter));
+	int ret;
+	
+	sniffInit();
+	ret = sniffPcap(1, file, filter);
+	sniffKill();
+	return (ret);
 }
