@@ -1,35 +1,58 @@
 /*
- * HTTPSniff v0.2 - HTTP Response Sniffer
+ * HTTPSniff v0.3 - HTTP Response Sniffer
  * stream.c
  * (C)2004-2005 Michael Poppitz
  *
- * Processes data stream - and currently does the http parsing.
+ * Processes data stream.
  *
  */
  
 #include <stdio.h>
 #include <string.h>
 
+#include <sys/stat.h>
+#include <sys/types.h>
+
+#include "config.h"
 #include "http.h"
 #include "stream.h"
 
-FILE *streamCreateFile(char *extension) {
+FILE *streamCreateFile(struct stream *s, char *extension) {
 	FILE *f;
+	char dir[RESOURCE_MAXSIZE + 1];
 	char name[0x100];
+	int i;
 
-	sprintf(name, "%.6u.%s", streamCount, extension);
-	streamCount++;
-
-	f = fopen(name, "w");
-	if (f) {
-		if (debug > 1) printf("streamCreateFile(): created %s\n", name);
+	// check if resource name is known and desired as file name, otherwise use plain numbering schemes
+	if (cfg.saveStructured && s->resource[0] != 0) {
+		// create directories
+		for (i = 0; s->resource[i] != 0; i++) {
+			if (s->resource[i] == '/') {
+				strncpy(dir, s->resource, i);
+				dir[i] = 0;
+				mkdir(dir, 0777);
+			}
+		}
+		// create file
+		f = fopen(s->resource, "w");
+		if (f && debug > 1)
+			printf("streamCreateFile(): created %s\n", s->resource);
+		s->resource[0] = 0;
 	} else {
-		perror("Could not open file for writing");
+		sprintf(name, "%.6u.%s", streamCount, extension);
+		streamCount++;
+		f = fopen(name, "w");
+		if (f && debug > 1)
+			printf("streamCreateFile(): created %s\n", name);
 	}
+	
+	if (!f)
+		perror("Could not open file for writing");
+	
 	return (f);
 }
 
-u_int32_t streamWriteFile(FILE *f, u_char *data, u_int32_t size, u_int32_t outstanding) {
+u_int32_t streamWriteFile(FILE *f, char *data, u_int32_t size, u_int32_t outstanding) {
 	u_int32_t written;
 	u_int32_t min;
 
@@ -42,8 +65,9 @@ u_int32_t streamWriteFile(FILE *f, u_char *data, u_int32_t size, u_int32_t outst
 }
 
 int streamDetect(struct stream *s, int force) {
-	u_char *start,*pos;
-	int isHeader, isChunkHeader;
+	char *start,*pos;
+	char *resource;
+	int isRequest, isResponse, isChunkHeader;
 
 	if (s->used == 0) 
 		return (0); // don't care for empty streams
@@ -61,11 +85,12 @@ int streamDetect(struct stream *s, int force) {
 	}
 
 	// determine if there is a complete header
-	isHeader = httpIsHeader(start);
+	isRequest = httpIsRequest(start);
+	isResponse = httpIsResponse(start);
 	isChunkHeader = httpIsChunkHeader(start);
 
 	// if not existing/complete and processing is not forced, return
-	if (!isHeader && !(s->type == HTTP_CHUNK && isChunkHeader) && !force)
+ 	if (!isRequest && !isResponse && !(s->type == HTTP_CHUNK && isChunkHeader) && !force)
 		return (0);
 
 	// unless its only a chunk header, the http type must be (re)detected
@@ -73,22 +98,22 @@ int streamDetect(struct stream *s, int force) {
 		s->type = httpType(start);
 
 	if (s->type == HTTP_UNKNOWN) {
-		s->file = streamCreateFile("raw");
+		s->file = streamCreateFile(s, "raw");
 		s->outstanding = 0xffffffff; /* FIX ME: outstanding should not be used for type unknown */
 		if (debug > 1) printf("streamDetect(): detected HTTP_UNKNOWN\n");
 
 	} else {
 		// create new output file if needed
-		if (!(s->type == HTTP_CHUNK && isChunkHeader) && s->type != HTTP_DISCARD) {
+		if (!(s->type == HTTP_CHUNK && isChunkHeader) && s->type != HTTP_DISCARD && s->type != HTTP_REQUEST) {
 			if (s->file)
 				fclose(s->file);
-			s->file = streamCreateFile(httpFileType(start));
+			s->file = streamCreateFile(s, httpFileType(start));
 		}
 
 		// determine expected/outstanding bytes
 		switch (s->type) {
 			case HTTP_SIMPLE:
-				s->outstanding = 0xffffffff; /* FIX ME: outstanding should not be used */
+				s->outstanding = 0xffffffff; /* FIX ME: outstanding should not be used here */
 				break;
 			case HTTP_KEEP:
 				s->outstanding = httpContentLength(start);
@@ -99,6 +124,15 @@ int streamDetect(struct stream *s, int force) {
 			case HTTP_DISCARD:
 				s->outstanding = 0;
 				break;
+			case HTTP_REQUEST:
+				if (s->twin) {
+					resource = httpResource(start);
+					if (debug > 1)
+						printf("streamDetect(): detected resource request: %s\n", resource);
+					strcpy(s->twin->resource, resource);
+				}
+				s->outstanding = 0;
+				break;
 			default:
 				printf("streamDetect(): Internal error: Invalid type\n");
 		}
@@ -111,10 +145,11 @@ int streamDetect(struct stream *s, int force) {
 		if (debug > 1) {
 			printf("streamDetect(): ");
 			switch (s->type) {
-				case HTTP_SIMPLE: printf("HTTP_SIMPLE"); break;
+				case HTTP_SIMPLE: printf("HTTP_SIMPLE\n"); break;
 				case HTTP_KEEP:	printf("HTTP_KEEP with %u total bytes expected\n", s->outstanding); break;
-				case HTTP_CHUNK: printf("HTTP_CHUNK"); break;
-				case HTTP_DISCARD: printf("HTTP_DISCARD"); break;
+				case HTTP_CHUNK: printf("HTTP_CHUNK\n"); break;
+				case HTTP_DISCARD: printf("HTTP_DISCARD\n"); break;
+				case HTTP_REQUEST: printf("HTTP_REQUEST\n"); break;
 			}
 		}
 	}
@@ -123,12 +158,13 @@ int streamDetect(struct stream *s, int force) {
 
 void streamOpen(struct stream *s) {
 	s->file = NULL;
+	s->resource[0] = 0;
 	s->used = 0;
 	s->outstanding = 0;
 	s->type = HTTP_NONE;
 }
 
-void streamWrite(struct stream *s, u_char *data, u_int32_t size) {
+void streamWrite(struct stream *s, char *data, u_int32_t size) {
 	size_t written;
 
 	if (debug > 2) printf("streamWrite(): %u bytes received\n", size);
