@@ -7,10 +7,12 @@
 
 struct stream {
 	FILE *file;
-	u_char cache[HEADER_MAXSIZE + 1];
+	char cache[HEADER_MAXSIZE + 1];
+	char resource[RESOURCE_MAXSIZE + 1];
 	u_int32_t outstanding;
 	u_int32_t used;
 	int type;
+	struct stream *twin;
 };
 
 int streamCount;
@@ -18,7 +20,7 @@ int streamCount;
 void streamInit();
 
 void streamOpen(struct stream *s);
-void streamWrite(struct stream *stream, u_char *data, u_int32_t size);
+void streamWrite(struct stream *stream, char *data, u_int32_t size);
 void streamClose(struct stream *s);
 
 #endif
